// feed is: { [ feedItem, ... ], secondsToNextUpdate: }
// feedItem is: { id, title, description, source, link, cssOverride } 
Sports247.Model.beginLoadFeed = function(serverUrl, teamName, appVersion, deviceId, successFn) {
	var fullUrl = serverUrl + "/feeds?team=" + teamName + "&deviceId=" + deviceId + "&appVersion=" + appVersion;
	Sports247.Utilities.Plist.beginLoadPlistData(fullUrl, function(feed) {
		feed.feedItems.reverse();
		for (var i = 0; i < feed.feedItems.length; i++) {
			// rename description to indicate html-ness of it
			feed.feedItems[i].descHtml = Sports247.Utilities.Html.cleanHtmlElement(feed.feedItems[i].description, true).html();
			delete feed.feedItems[i].description;

			// short description text
			var descElement = Sports247.Utilities.Html.cleanHtmlElement(feed.feedItems[i].descHtml);
			var descText = descElement.text();
			feed.feedItems[i].shortDesc = Sports247.Utilities.String.clipText(descText, 100);
			
			// index
			feed.feedItems[i].feedItemIndex = i;

			// make sure text is clean of html
			feed.feedItems[i].title = Sports247.Utilities.Html.cleanHtmlElement(feed.feedItems[i].title).text();
			feed.feedItems[i].source = Sports247.Utilities.Html.cleanHtmlElement(feed.feedItems[i].source).text();
		}
		
		successFn(feed);
	});
};

Sports247.Model.beginLoadSchedule = function(serverUrl, teamName, successFn) {
	var fullUrl = serverUrl + "/cal?team=" + teamName;
	Sports247.Utilities.Plist.beginLoadPlistData(fullUrl, function(schedule) {
		Sports247.Model.Internal.ProcessSchedule(schedule);
		successFn(schedule);
	});
}

Sports247.Model.Internal = {};

Sports247.Model.Internal.ProcessSchedule = function(schedule) {
	if (schedule && schedule.games && schedule.games.length > 0) {
		var dateFormat = "yyyyMMddTHHmmssZ";
		for (var i = 0; i < schedule.games.length; i++) {				
			// parse the dates
			schedule.games[i].start = Date.parseString(schedule.games[i].start, dateFormat);
			schedule.games[i].end = Date.parseString(schedule.games[i].end, dateFormat);
			if (!schedule.games[i].start || !schedule.games[i].end) {
				schedule = null;
				break;
			}

			// Adjust for timezone
			schedule.games[i].start.add('m', -schedule.games[i].start.getTimezoneOffset());
			schedule.games[i].end.add('m', -schedule.games[i].end.getTimezoneOffset());
			
			// Store friendly text
			schedule.games[i].startDateText = Sports247.Utilities.Date.niceDateString(schedule.games[i].start, 'NNN d');
			schedule.games[i].startTimeText = schedule.games[i].start.format('h:mm a');
			
			// make sure text is clean of html
			schedule.games[i].description = Sports247.Utilities.Html.cleanHtmlElement(schedule.games[i].description).text();
			schedule.games[i].summary = Sports247.Utilities.Html.cleanHtmlElement(schedule.games[i].summary).text();
			
			// Add index
			schedule.games[i].gameIndex = i;
		}
		
		// divide games up to previous and current/future
		var now = new Date();
		var dateResults = Sports247.Model.Internal.SeparateByDate(now, schedule.games, function(game) { return game.end; });
		schedule.previousGames = dateResults[0];
		schedule.currentAndNextGames = dateResults[1];
	}
}

// returns [[before date], [on/after date]]
Sports247.Model.Internal.SeparateByDate = function(date, items, fnDateOfItem) {
	var results = [[], []];
	for (var i = 0; i < items.length; i++) {
		var dateOfItem = fnDateOfItem(items[i]);
		if (dateOfItem.isBefore(date)) {
			results[0].push(items[i]);
		} else {
			results[1].push(items[i]);
		}
	}
	
	return results;
}