function testUnit_runUnitTests() {
	Sports247.UnitTests.start = function() {
		console.log("running unit tests...");
		try {
			Sports247.UnitTests.Utilities.start();
			Sports247.UnitTests.Model.start();
			Sports247.UnitTests.Html.start();
			Sports247.UnitTests.AppBehaviors.start();
		} catch (err) {
			console.log("UNIT TEST FAIL!! " + err);
			alert("UNIT TEST FAIL: " + err);
			throw "unit test fail";
		}
		console.log("unit test pass");
	};

	Sports247.UnitTests.Utilities = {};
	Sports247.UnitTests.Utilities.start = function() {
		Sports247.UnitTests.Utilities.CleanHtmlTest();
		Sports247.UnitTests.Utilities.ClipStringText();
		Sports247.UnitTests.Utilities.NiceDateTest();
	};
	Sports247.UnitTests.Utilities.CleanHtmlTest = function() {
		var tests = [
			["hi", "hi", false],
			["<b>hi</b>", "hi", false],
			["<b>hi</b>", "<b>hi</b>", true],
			["yo<ma>ma</ma>", "yoma", false],
			["yo<ma>ma</ma>", "yoma", true],
			["hi<script type='text/javascript'></script>bye", "hibye", false],
			["hi<script type='text/javascript'>blahb();</script>bye", "hibye", true],
			["hi<script type='text/javascript'>blahb();</script>bye", "hibye", false],
			["hola<b>hi</b>and<script type='text/javascript'>aoirhgoi</script>bye<img src='b'>", "holahiandbye", false],
			["hola<b>hi</b>and<script type='text/javascript'>aoirhgoi</script>bye<img src='b'>", "hola<b>hi</b>andbye<img src=\"b\">", true],
			["hola<b>hi</b>and<script type='text/javascript'>aoirhgoi</script>bye<img    src='b'>", "hola<b>hi</b>andbye<img src=\"b\">", true],
			["hola<p><!--<div/>-->hi</p>", "hola<p>hi</p>", true],
			["hola<p>hi</p", "hola<p></p>", true],	// ideally we'd keep the 'hi' but in this error case, we can't guarantee it's not script easily
		];

		compareTest("CleanHtmlTest", tests, function(original, keepFormatting) {
			var resultElement = Sports247.Utilities.Html.cleanHtmlElement(original, keepFormatting, keepFormatting);
			var result = resultElement.html();
			return result;
		});
	};
	
	Sports247.UnitTests.Utilities.ClipStringText = function() {
		var tests = [
			["hello there", "hello there", 11],
			["hello there", "hello", 10],
			["hello there", "hello", 7],
			["hello there", "hello", 6],
			["hello there", "hello", 5],
			["hello there", "h...", 4],
			["hello there", "hel", 3],
			["hello there", "he", 2],
			["hello there", "h", 1],
			["a b c d", "a b", 3],
			["a b c d", "a b c", 6]
		];

		compareTest("ClipStringText", tests, function(original, maxChars) {
			var result = Sports247.Utilities.String.clipText(original, maxChars);
			return result;
		});
	};
	
	Sports247.UnitTests.Utilities.NiceDateTest = function() {
		var today = new Date();
		var tests = [
			[today, "Today", "nope"],
			[new Date(today.getTime()).add('d', -1), "Yesterday", "nope"],
			[new Date(today.getTime()).add('d', 1), "Tomorrow", "nope"],
			[new Date(today.getTime()).add('d', -2), "nope", "nope"],
			[new Date(today.getTime()).add('d', 2), "nope", "nope"]
		];

		compareTest("NiceDateTest", tests, function(date, dateFormat) {
			var result = Sports247.Utilities.Date.niceDateString(date, dateFormat);
			return result;
		});
	};
	
	Sports247.UnitTests.Model = {};
	Sports247.UnitTests.Model.start = function() {
		Sports247.UnitTests.Model.DateSeparationTest();
	};
	Sports247.UnitTests.Model.DateSeparationTest = function() {
		// try 1 item that is the current date
		var now = new Date();
		var result = Sports247.Model.Internal.SeparateByDate(now, [now], function(d) { return d;});
		assert(result.length == 2);
		assert(result[0].length === 0);
		assert(result[1].length == 1);
		assert(result[1][0] == now);
		
		// try 1 item before current date
		var beforeItem = new Date();
		beforeItem.add('h', -1);
		result = Sports247.Model.Internal.SeparateByDate(now, [beforeItem], function(d) { return d;});
		assert(result.length == 2);
		assert(result[0].length == 1);
		assert(result[1].length === 0);
		assert(result[0][0] == beforeItem);

		// try 1 item after current date
		var afterItem = new Date();
		afterItem.add('h', 2);
		result = Sports247.Model.Internal.SeparateByDate(now, [afterItem], function(d) { return d;});
		assert(result.length == 2);
		assert(result[0].length === 0);
		assert(result[1].length == 1);
		assert(result[1][0] == afterItem);
		
		// try 1 of each (unsorted)
		result = Sports247.Model.Internal.SeparateByDate(now, [afterItem, beforeItem], function(d) { return d;});
		assert(result.length == 2);
		assert(result[0].length == 1);
		assert(result[1].length == 1);
		assert(result[0][0] == beforeItem);
		assert(result[1][0] == afterItem);
		
		// try 1 of each (sorted)
		result = Sports247.Model.Internal.SeparateByDate(now, [beforeItem, afterItem], function(d) { return d;});
		assert(result.length == 2);
		assert(result[0].length == 1);
		assert(result[1].length == 1);
		assert(result[0][0] == beforeItem);
		assert(result[1][0] == afterItem);
	};
	
	Sports247.UnitTests.Html = {};
	Sports247.UnitTests.Html.start = function() {
		Sports247.UnitTests.Html.validatePageStructure();
		Sports247.UnitTests.Html.validateElementIdsAndClasses();
		Sports247.UnitTests.Html.validateContent();
	};
	
	Sports247.UnitTests.Html.validateElementIdsAndClasses = function() {
		var elementIds = Sports247.App.Debug.getElementIds();
		var isTested = false;
		for (var id in elementIds) {
			assert($(elementIds[id]).length == 1, "unable to find on page: " + elementIds[id]);
			isTested = true;
		}
		assert(isTested, "no element ids to validate");
		
		var classes = Sports247.App.Debug.getCssClasses();
		isTested = false;
		for (var cssClass in classes) {
			assert($(classes[cssClass]).length >= 1, "unable to find on page: " + classes[cssClass]);
			isTested = true;
		}
		assert(isTested, "no css classes to validate");
	};
	
	Sports247.UnitTests.Html.validatePageStructure = function() {
		// FUTURE: validate this is true on page changes with content
		// FUTURE: verify scroll bar only added once
		$('.page').each(function(i, pageElement) {
			// Validate 0 or 1 header
			var headerElement = $(pageElement).find('.header');
			assert(headerElement.length === 0 || headerElement.length == 1, "too many html headers in a page");
			if (headerElement.length == 1) {
				assert(headerElement.parent()[0] === pageElement, "header is not child of page");
				assert($(pageElement).find('.header ~ .contentOuter').length == 1, 'contentOuter does not follow header');
			}

			var footerElement = $(pageElement).find('.footer');
			assert(footerElement.length === 0 || footerElement.length == 1, "too many html footers in a page");
			if (footerElement.length == 1) {
				assert(footerElement.parent()[0] === pageElement, "footerElement is not child of page");
				assert($(pageElement).find('.footer ~ .contentOuter').length == 1, 'contentOuter does not follow footer');
			}

			// Validate 1 content outer
			var contentOuter = $(pageElement).find('.contentOuter');
			assert(contentOuter.length == 1, "no contentOuter on page");
			assert(contentOuter.parent()[0] === pageElement, "contentOuter is not child of page");

			// Validate 1 content
			var content = contentOuter.find('.content');
			assert(content.length == 1, "no content in contentOuter");
			assert(content.parent()[0] === contentOuter[0], "content is not child of contentOuter");
			
			// Validate only 1 extra child on content outer (iScroll)
			var iScrollCheck = (contentOuter.children().length > 1);
			if (iScrollCheck) {
				assert(contentOuter.children().length == 2, "contentOuter has too many children");
				assert(contentOuter.css('overflow-x') == 'hidden', "contentOuter not setup for iScroll properly");
				assert(contentOuter.css('overflow-y') == 'hidden', "contentOuter not setup for iScroll properly");
			}
		});
	};
	
	Sports247.UnitTests.Html.validateContent = function() {
		var appReviews = $('.reviewApp');
		var appReviewLinks = $('a.reviewApp');
		assert(appReviews.length == appReviewLinks.length, ".reviewApp that's not an <a>");
		assert(appReviewLinks.length > 0, "no app review links?");
		var appReviewHref = appReviewLinks[0].href;
		var appReviewHrefStart = "market://details?id=com.sparetimeventures.";
		assert(appReviewHref.substr(0,appReviewHrefStart.length) == appReviewHrefStart, "bad .reviewApp link");
		$.each(appReviewLinks, function(i, appReviewLink) {
			assert(appReviewLink.href == appReviewHref, "inconsistent app review link");
		});
		var marketLinks = $('a[href^=market]');
		assert(marketLinks.length == appReviewLinks.length);
	};
	
	Sports247.UnitTests.AppBehaviors = {};
	Sports247.UnitTests.AppBehaviors.start = function() {
		Sports247.UnitTests.AppBehaviors.requestReviewTest();
		Sports247.UnitTests.AppBehaviors.menuButtonTest();
		Sports247.UnitTests.AppBehaviors.notificationsTest();
		Sports247.UnitTests.AppBehaviors.notificationsUiTest();
	};
	Sports247.UnitTests.AppBehaviors.requestReviewTest = function() {
		var runKeys = Sports247.App.Debug.getRunInfoKeys();
		runTestWithFreshSettings(runKeys, function() {
			// Boot the app several times -- nothing should happens since same date
			var currentPageId = Sports247.UI.activePage().attr("id");
			var pageChanged;
			for (var i = 0; i < 100; i++) {
				pageChanged = Sports247.App.setupAppReviewRequest();
				assert(!pageChanged, "unexpected page change");
				assert(Sports247.Utilities.LocalStorage.getDate(runKeys.dateFirstRunKey).equalsIgnoreTime(new Date()), "first run date not set right");
				assert(Sports247.Utilities.LocalStorage.getInt(runKeys.useCountKey) == i + 1, "run count not set (first boot)");
				assert(Sports247.Utilities.LocalStorage.getBool(runKeys.dontCheckKey) === undefined, "dont check set wrong");
				assert(Sports247.Utilities.LocalStorage.getDate(runKeys.dateLastPromptKey) === undefined, "dateLastPromptKey set wrong");
				assert(Sports247.UI.activePage().attr("id") == currentPageId, "unexpected page change");
			}
			
			// Now move the date back until we get a page change
			var reviewRequested = false;
			for (i = 1; i < 10; i++) {
				localStorage[runKeys.dateFirstRunKey] = new Date().clearTime().add('d', -i);
				pageChanged = Sports247.App.setupAppReviewRequest();
				if (Sports247.UI.activePage().attr("id") != currentPageId) {
					assert(pageChanged);
					assert(Sports247.UI.activePage().attr("id") == "reviewRequestPage");
					assert(i > 3, "not enough days before review requested");
					reviewRequested = true;
					break;
				}
			}
			assert(reviewRequested, "never got a review request");

			// Now that the page is up, make sure we don't get out of it automatically
			assert(Sports247.UI.activePage().attr("id") == "reviewRequestPage");
			Sports247.UI.changePage("#" + currentPageId);
			assert(Sports247.UI.activePage().attr("id") == "reviewRequestPage", "reviewRequestPage forced off");

			// Now check that we keep prompting the user, but only once a day
			for (i = 1; i < 10; i++) {
				pageChanged = Sports247.App.setupAppReviewRequest();
				assert(!pageChanged, "page change when booting app on same day");
			}
			
			// Fake last prompt date to yesterday and make sure we prompt
			localStorage[runKeys.dateLastPromptKey] = new Date().add('d', -1);
			pageChanged = Sports247.App.setupAppReviewRequest();
			assert(pageChanged, "page should have changed");
			
			// Now answer review app and make sure we never prompt again and that we left the page
			assert(Sports247.Utilities.LocalStorage.getBool(runKeys.dontCheckKey) === undefined, "dont-check shouldn't be set here");
			assert(Sports247.UI.activePage().attr("id") != currentPageId, "should be in review page");
			cssClasses = Sports247.App.Debug.getCssClasses();
			$(cssClasses.reviewApp).click();
			assert(Sports247.Utilities.LocalStorage.getBool(runKeys.dontCheckKey) === true, "clicking on app review should set dontCheckKey");
			assert(Sports247.UI.activePage().attr("id") == currentPageId, "should have went back to original page");

			localStorage[runKeys.dateLastPromptKey] = new Date().add('d', -4);
			pageChanged = Sports247.App.setupAppReviewRequest();
			assert(!pageChanged, "page shouldn't have changed");
			
			// Make sure 'no' has same affect
			localStorage.removeItem(runKeys.dontCheckKey);
			assert(Sports247.Utilities.LocalStorage.getBool(runKeys.dontCheckKey) === undefined, "dont-check shouldn't be set here");
			$(cssClasses.neverReviewApp).click();
			assert(Sports247.Utilities.LocalStorage.getBool(runKeys.dontCheckKey) === true, "clicking on app review should set dontCheckKey");
		});
	};
	Sports247.UnitTests.AppBehaviors.menuButtonTest = function() {
		assert($("#morePage.active-page").length === 0, "more page already active");
		Sports247.App.handleMenuButton();
		assert($("#morePage.active-page").length == 1, "more page not activated");
		Sports247.App.handleMenuButton();
		assert($("#morePage.active-page").length == 1, "more page not activated after menu button");
	};
	
	Sports247.UnitTests.AppBehaviors.notificationsTest = function() {
		var runKeys = Sports247.App.Debug.getNotificationKeys();
		runTestWithFreshSettings(runKeys, function() {

			Sports247.App.Debug.refreshNotificationTime();

			// Make sure we're starting from scratch
			assert(plugins.localNotification.debugGetNotifications().length === 0, "starting with notifications");
			assert(Sports247.App.getScheduleData() === null, "starting with schedule data");
			assert(Sports247.App.tryUpdatingNotifications() === false, "able to update notifications without schedule data");

			// Add some schedule items
			var schedulePlist = Sports247.Model.TestData.getSchedulePlist();
			var scheduleData = Sports247.Utilities.Plist.parsePlist(schedulePlist);
			Sports247.Model.Internal.ProcessSchedule(scheduleData);
			Sports247.App.Debug.forceSchedule(scheduleData, function() {
				assert(Sports247.App.tryUpdatingNotifications() === true, "unable to process schedule");
				assert(plugins.localNotification.debugGetNotifications().length === 0, "notifications without user setting");
			});

			// set reminder
			Sports247.App.turnOnNotifications();
			Sports247.App.Debug.forceSchedule(scheduleData, function() {
				assert(Sports247.App.tryUpdatingNotifications() === true, "unable to process schedule");
				assert(plugins.localNotification.debugGetNotifications().length == scheduleData.currentAndNextGames.length, "notifications not set");
				assert(plugins.localNotification.debugGetNotifications()[0].date != scheduleData.currentAndNextGames[0].start, "notification date set exactly to game time");
				assert(plugins.localNotification.debugGetNotifications()[0].date.equalsIgnoreTime(scheduleData.currentAndNextGames[0].start), "notification day is different than game day");

				var msg = plugins.localNotification.debugGetNotifications()[0].message.split("\r\n");
				assert(msg.length == 2);
				assert(msg[0] == scheduleData.currentAndNextGames[0].summary, "notification summary is wrong");
				assert(msg[1] == scheduleData.currentAndNextGames[0].description, "notification description is wrong");
			});

			// test reset to 0
			Sports247.App.turnOffNotifications();
			Sports247.App.Debug.forceSchedule(scheduleData, function() {
				assert(Sports247.App.tryUpdatingNotifications() === true, "unable to process schedule");
				assert(plugins.localNotification.debugGetNotifications().length === 0, "notifications not removed");
			});
		});

		Sports247.App.Debug.refreshNotificationTime();
	};

	Sports247.UnitTests.AppBehaviors.notificationsUiTest = function() {
		var selectorToggle = Sports247.App.Debug.getCssClasses().gameNotifyToggle;
		var runKeys = Sports247.App.Debug.getNotificationKeys();

		var turnOffText = "Turn off Game Reminders";
		var turnOnText = "Remind me of upcoming games!";

		runTestWithFreshSettings(runKeys, function() {
			Sports247.App.Debug.refreshNotificationTime();

			var schedulePlist = Sports247.Model.TestData.getSchedulePlist();
			var scheduleData = Sports247.Utilities.Plist.parsePlist(schedulePlist);
			Sports247.Model.Internal.ProcessSchedule(scheduleData);
			Sports247.App.Debug.forceSchedule(scheduleData, function() {

				assert(plugins.localNotification.debugGetNotifications().length === 0, "notifications set when shouldn't be");
				assert($(selectorToggle).text() == turnOnText, "initial state wrong");

				$(selectorToggle).click();
				assert($(selectorToggle).text() == turnOffText, "reminder state wrong");
				assert(plugins.localNotification.debugGetNotifications().length > 0, "notifications set when shouldn't be");

				$(selectorToggle).click();
				assert($(selectorToggle).text() == turnOnText, "reminder state wrong");
				assert(plugins.localNotification.debugGetNotifications().length === 0, "notifications should be off");
			});
		});

		Sports247.App.Debug.refreshNotificationTime();
	};
	
	
	// where tests is an array of [original, expected result, params after original]
	function compareTest(testName, tests, xformFn) {
		for (var i in tests) {
			var xformArgs = $.merge([tests[i][0]], tests[i].slice(2, tests.length));
			var result = xformFn.apply(this, xformArgs);
			assert(result == tests[i][1], testName + "! original:" + tests[i][0] + ", " + xformArgs[1] + " -- got back: " + result + ", expected: " + tests[i][1]);
		}
	}

	function runTestWithFreshSettings(runKeys, testFn) {
		var savedRunInfo = {};
		for (var runKey in runKeys) {
			var keyValue = runKeys[runKey];
			if (localStorage[keyValue] !== undefined) {
				savedRunInfo[keyValue] = localStorage[keyValue];
			}
		}

		try {
			// Clear out all keys
			for (var runKey in runKeys) {
				var keyValue = runKeys[runKey];
				if (localStorage[keyValue] !== undefined) {
					localStorage.removeItem(keyValue);
				}
			}

			testFn();
		} catch(err) {
			throw err;
		} finally {
			// Restore run keys
			for (var runKey in runKeys) {
				var keyValue = runKeys[runKey];
				if (undefined === savedRunInfo[keyValue]) {
					localStorage.removeItem(keyValue);
				} else {
					localStorage[keyValue] = savedRunInfo[keyValue];
				}
			}
		}
	}

	function assert(bool, msg) {
		if (!bool) {
			if (!msg) {
				msg = "fail";
			}
			throw msg;
		}
	}

	$(document).ready(Sports247.UnitTests.start());
}