(function() {
	var _activePageClass = 'active-page';
	var _pageLockCountAttr = 'pageLock';

	var _pageTransitionTime = 200;
	
	function animatePages(cssPropertyAndShowValue, prevPage, prevPageToAnimate, prevPageHide, newPage, newPageToAnimate, newPageHide) {
		// FUTURE: rather than blowing away values that may already exist, we should animate to them
		var cssPrevPageHide = $.extend({}, cssPropertyAndShowValue);
		var cssNewPageHide = $.extend({}, cssPropertyAndShowValue);
		for (var k in cssPrevPageHide) { cssPrevPageHide[k] = prevPageHide; }
		for (var k in cssNewPageHide) { cssNewPageHide[k] = newPageHide; }
		
		prevPageToAnimate.animate(
			cssPrevPageHide,
			_pageTransitionTime,
			function() {
				prevPage.removeClass(_activePageClass);
				prevPageToAnimate.css(cssPropertyAndShowValue);
			});
		
		newPageToAnimate.css(cssNewPageHide);
		newPage.addClass(_activePageClass);
		newPageToAnimate.animate(cssPropertyAndShowValue, _pageTransitionTime, function() { newPageToAnimate.css(cssPropertyAndShowValue); });
	}
	
	function matchPageToHash(page, transition) {
		var pageElement = $(page);
		if (pageElement.hasClass('page')) {
			var prevPage = $('.'+_activePageClass);
			
			// Avoid changing to same pages
			if (prevPage.length == 1 && pageElement.length == 1 && prevPage[0] == pageElement[0]) {
				return;
			}

			pageElement.trigger("pagebeforechange", [prevPage, pageElement]);
	
			var windowWidth = $(window).width();
			var windowWidthPx = windowWidth + "px";
			var negWindowWidthPx = "-" + windowWidth + "px";
			switch (transition) {
				case 'fade':
					animatePages({opacity: 1.0}, prevPage, prevPage.children(), 0.0, pageElement, pageElement.children(), 0.0);
					break;
				
				case 'slide-from-right':
					animatePages({'margin-left': 0}, prevPage, prevPage.children(), negWindowWidthPx, pageElement, pageElement.children(), windowWidthPx);
					break;

				case 'slide-from-left':
					animatePages({'margin-left': 0}, prevPage, prevPage.children(), windowWidthPx, pageElement, pageElement.children(), negWindowWidthPx);
					break;
				
				default:
					prevPage.removeClass(_activePageClass);
					pageElement.addClass(_activePageClass);
					break;
			}

			pageElement.trigger('pageafterchange', pageElement);
		}
	}
	
	function handleNewHash(newHash) {
		if (getPageLockCount() > 0)
			return false;

		var hash = /^#[^?&]+/.exec(newHash);
		if (!hash || hash.length != 1) {
			return;
		}
		hash = hash[0];
		newPageElement = $(hash);
		
		var transition = /[\&\?]page-transition=([^\&\?]+)/.exec(newHash);
		if (transition && transition.length == 2) {
			transition = transition[1];
		}

		if (newPageElement.length == 1) {
			matchPageToHash(newPageElement[0], transition);
		} else {
			$(document).trigger('virtualpageactivation', hash);
		}

		return true;
	}
	
	Sports247.UI.handleHashChange = function(event) {
		handleNewHash(window.location.hash);
	};
	
	Sports247.UI.activePage = function() {
		return $('.'+_activePageClass);
	};
	
	Sports247.UI.changePage = function(page) {
		window.location = page;
		handleNewHash(page);
	};

	function getPageLockCount() {
		var pageLockCount = parseInt(Sports247.UI.activePage().attr(_pageLockCountAttr));
		if (!pageLockCount) {
			pageLockCount = 0;
		}
		return pageLockCount;
	}
	function incrementPageLockCount(increment) {
		var pageLockCount = getPageLockCount();
		pageLockCount += increment;
		pageLockCount = Sports247.UI.activePage().attr(_pageLockCountAttr, pageLockCount);
	}
	Sports247.UI.lockCurrentPage = function() {
		incrementPageLockCount(1);
	};

	Sports247.UI.unlockCurrentPage = function() {
		incrementPageLockCount(-1);
	};

	// followed by set of states where each state is [label, fnOnClick]
	Sports247.UI.setupToggleButton = function(selector, states, changeEvents, fnCurrentState) {
		$(selector).text(states[fnCurrentState()][0]);
		$(selector).bind("click", function() {
			states[fnCurrentState()][1]();
		});
		$(selector).bind(changeEvents.join(" "), function() {
			$(selector).text(states[fnCurrentState()][0]);
		});

	};
		
	// handle page nav
	$(window).bind('hashchange', Sports247.UI.handleHashChange);

	// handle styling for touch
	var touchCssSelectors = ['a.button', '.radioTabSet a'];
	$.each(touchCssSelectors, function(i, cssSelector) {
		$(document).delegate(cssSelector, {
			'touchstart': function() { this.setAttribute("touched", ""); },
			'mousedown': function() { this.setAttribute("touched", ""); },

			'touchmove': function() { this.removeAttribute("touched"); },
			'mouseup': function() { this.removeAttribute("touched"); },
			'touchend': function() { this.removeAttribute("touched"); }
		});
	});

})();
