/* HTML Utilities */
Sports247.Utilities.Html = {};
Sports247.Utilities.Html.cleanHtmlElement = function(html, keepFormatting, logUnhandledTags) {
	// FUTURE: check tags in real stories 
	
	// true: formatting tag
	// array: formatting tag with list of attributes to keep
	// false: invalid tag... remove it, the end tag, and everything in-between
	// undefined: unknown tag
	var tagInfo = {
		/* simple formatting tags */
		'b': true, '/b' : true,
		'p' : true, '/p' : true, 'p/' : true,
		'em': true, '/em': true,
		'p': true, '/p' : true,
		'div': true, '/div': true,
		'table': true, '/table': true,
		'tbody': true, '/tbody': true,
		'tr': true, '/tr': true,
		'td': true, '/td': true,
		'th': true, '/th': true,
		'br/': true, 'br': true,
		'ul': true, '/ul': true,
		'li': true, '/li': true,
		'ol': true, '/ol': true,
		'i': true, '/i': true,
		'h1': true, '/h1': true,
		'h2': true, '/h2': true,
		'h3': true, '/h3': true,
		'h4': true, '/h4': true,
		'h5': true, '/h5': true,
		'blockquote': true, '/blockquote': true,
		'strong': true, '/strong': true,
		'span': true, '/span': true,
		'strike': true, '/strike': true,
		'cite': true, '/cite': true,
		'small': true, '/small': true,
		
		/* formatting tags with attributes */
		'img': ['src\\s*=\\s*["\'][^"\']+["\']'],
		'font': ['size\\s*=\\s*["\'][^"\']+["\']'],
		'/font': true,

		/* tags to remove completely */
		'script': false,
	};
	
	var knownUnhandledTags = {
		'a': true, '/a': true,
		'/script': true,
	}
	
	var text = '';
	var tagsToReinsert = [];	// each entry is ['tag', index to insert in text]. note that the index is not adjusted for other tag inserts
	var tagsUnhandled = [];
	while (true) {
		var itagOpen = html.indexOf("<");
		if (itagOpen >= 0) {
			var itagClose = -1;
			if (html.substring(itagOpen+1, itagOpen+4) == "!--") {
				// handle comments specially since they tend to have html in them
				// and we're better off never looking at anything in the tag
				itagClose = html.indexOf("-->");
				if (itagClose >= 0) {
					itagClose += 2;
				}
			} else {
				itagClose = html.indexOf(">");
			}
			if (itagClose > itagOpen) {
				var currentTagAndAttributes = html.substring(itagOpen, itagClose+1);
				var currentTag = currentTagAndAttributes.match(/<\s*([^\s]+).*>/);
				if (currentTag) {
					currentTag = currentTag[1];
				}
				text += html.substring(0, itagOpen);
				html = html.substring(itagClose + 1, html.length);

				if (tagInfo[currentTag] == true && keepFormatting) {
					tagsToReinsert.push([currentTag, text.length]);
				} else if (tagInfo[currentTag] && tagInfo[currentTag].constructor.name == "Array" && keepFormatting) {
					var tagWithAttributes = [currentTag];
					for (var j in tagInfo[currentTag]) {
						var regexAttr = new RegExp(tagInfo[currentTag][j]);
						var attr = currentTagAndAttributes.match(regexAttr);
						if (attr) {
							tagWithAttributes.push(attr);
						}
					}
					tagsToReinsert.push([tagWithAttributes.join(" "), text.length]);
				} else if (tagInfo[currentTag] == false) {
					var itagEndOpen = html.search(new RegExp("<.*" + currentTag + ">"));
					if (itagEndOpen >= 0) {
						html = html.substring(itagEndOpen, html.length);
					}
				}
				else {
					if (!knownUnhandledTags[currentTag]) {
						tagsUnhandled.push(currentTagAndAttributes);
					}
				}
			} else {
				// we have a situation like html = "/p> ... <p"
				// we'll either move forward ahead so html = "<p" and throw away what was before
				// or if there's no closing tag so itagOpen == 0 (no progress can happen this way),
				//       we'll toss it all away
				console.log("ERROR PARSING HTML SECTION: " + html.substring(0, itagOpen));
				if (itagOpen > 0) {
					html = html.substring(itagOpen, html.length);
				} else {
					html = "";
				}
			}
		} else {
			text += html;
			break;
		}
	}
	
	/* make it all text to be extra safe */
	var e = $("<div/>");
	e.text(text);
	
	// validate that the conversion worked. if so, re-insert formatting back. if not, just return what we
	// have to be safe
	if (e.text().length == text.length) {
		for (var i = tagsToReinsert.length - 1; i >= 0; i--) {
			text = Sports247.Utilities.String.splice(text, tagsToReinsert[i][1], 0, "<" + tagsToReinsert[i][0] + ">");
		}
		e.html(text);
	}
	
	if (logUnhandledTags && tagsUnhandled.length > 0) {
		console.log("UNHANDLED TAGS: " + tagsUnhandled.join("\r"));
	}

	return e;
}


/* String Utilities */
Sports247.Utilities.String = {};
Sports247.Utilities.String.splice = function( string, idx, rem, s ) {
    return (string.slice(0,idx) + s + string.slice(idx + Math.abs(rem)));
};

Sports247.Utilities.String.clipText = function(text, maxChars) {
	var ellipses = '...';
	if (text.length <= maxChars) {
		return text;
	}
	
	if (maxChars <= ellipses.length) {
		return text.slice(0, maxChars);
	}
	
	for (var i = maxChars; i >= 0; i--) {
		if (text[i] == ' ' || text[i] == '\r' || text[i] == '\n') {
			return text.slice(0, i);
		}
	}
	
	return Sports247.Utilities.String.splice(text, maxChars - ellipses.length, text.length, ellipses);
}



/* PList Utilities */
Sports247.Utilities.Plist = {};
Sports247.Utilities.Plist.beginLoadPlistData = function(url, successFn) {
	var maxAttempts = 2;
	function tryLoad(attempt) {
		$.ajax({
			url: url,
			timeout: 5000,
			success: function(data) {
				var parsedData = Sports247.Utilities.Plist.parsePlist(data);
				if (!parsedData) {
					failLoad(attempt, null, null, null);
				} else {
					successFn(parsedData);
				}
			},
			error: function(jqXHR, textStatus, errorThrown) {
				failLoad(attempt, jqXHR, textStatus, errorThrown);
			}
		});
	}
	
	function failLoad(attempt, jqXHR, textStatus, errorThrown) {
		if (attempt < maxAttempts) {
			tryLoad(attempt + 1);
		} else {
			// FUTURE: this if sucks for testing
			if (navigator.notification) {
				navigator.notification.alert(
					"Sorry we were unable to update the news. Please check your connection and try again",
					tryLoad,
					Sports247.App.fullTeamName(),
					"Try again");
			} else {
				alert("Sorry we were unable to update the news. Please check your connection and try again");
				tryLoad();
			}
		}
	}
	
	tryLoad(1);
};

Sports247.Utilities.Plist.parsePlist = function(plistXml) {
	
	function parsePlistDict(element, currentDict) {
		var dict = {};
		var children = $(element).children();
		for (var i = 0; i < children.length; i+=2) {
			if (children[i].nodeName == "key") {
				var keyName = $(children[i]).text();
				var value = parsePlistItem(children[i+1]);
				dict[keyName] = value;
			} else {
				throw "where's they key?";
			}
		}
		
		return dict;
	}
	
	function parsePlistItem(element) {
		var itemValue = null;
		switch (element.nodeName) {
			case "array":
				itemValue = [];
				var arrayItems = $(element).children();
				for (var j = 0; j < arrayItems.length; j++) {
					var childValue = parsePlistItem(arrayItems[j]);
					itemValue.push(childValue);
				}
				break;
			case "string":
				itemValue = $(element).text();
				break;
			case "integer":
				itemValue = parseInt($(element).text());
				break;
			case "date":
				// FUTURE: handle this correctly
				itemValue = $(element).text();
				break;
			case "dict":
				itemValue = parsePlistDict(element);
				break;
			default:
				throw "I dont know what type of value this is: " + element.nodeName + " (" + $(element).text() + ")";
				break;
		}
		return itemValue;
	}
	
	try {
		var plistDom = $.parseXML(plistXml);
		var plistRoot = plistDom.getElementsByTagName("plist")[0];
		var plistDict = $(plistRoot).children()[0];
		var plistJson = parsePlistDict(plistDict);
		return plistJson;
	} catch(err) {
		console.log(err);
	}
	
	return null;
};

Sports247.Utilities.Plist.getPlistHeader = function() {
	var plist = '<?xml version="1.0" encoding="UTF-8"?>';
	plist += '<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">';
	plist += '<plist version="1.0">';
	return plist;
};

Sports247.Utilities.Plist.getPlistFooter = function() {
	return '</plist>';
};

Sports247.Utilities.Plist.getPlistDict = function(key, value) {
	return Sports247.Utilities.Plist.getPlistItem(key, 'dict', value);
};

Sports247.Utilities.Plist.getPlistArray = function(key, value) {
	return Sports247.Utilities.Plist.getPlistItem(key, 'array', value);
};

Sports247.Utilities.Plist.getPlistString = function(key, value) {
	return Sports247.Utilities.Plist.getPlistItem(key, 'string', value);
};

Sports247.Utilities.Plist.getPlistInteger = function(key, value) {
	return Sports247.Utilities.Plist.getPlistItem(key, 'integer', value);
};

Sports247.Utilities.Plist.getPlistItem = function(key, type, value) {
	var plistItem = '';
	if (key) {
		plistItem += '<key>' + key + '</key>';
	}
	
	plistItem += '<' + type + '>';
	plistItem += value;
	plistItem += '</' + type + '>';
	
	return plistItem;
};


/* Date utilities */
Sports247.Utilities.Date = {};
Sports247.Utilities.Date.niceDateString = function(date, format) {
	var today = new Date();
	today.clearTime();
	
	var yesterday = new Date(today.getTime());
	yesterday.add('d', -1);

	var tomorrow = new Date(today.getTime());
	tomorrow.add('d', 1);
	
	var dayAfterTomorrow = new Date(tomorrow.getTime());
	dayAfterTomorrow.add('d', 1);
	
	var ranges = [
		[yesterday,	today,				"Yesterday"],
		[today,		tomorrow,			"Today"],
		[tomorrow,	dayAfterTomorrow,	"Tomorrow"]
	];
	
	for (var i = 0; i < ranges.length; i++) {
		if (date.isAfter(ranges[i][0]) && date.isBefore(ranges[i][1])) {
			return ranges[i][2];
		}
	}
	
	return date.format(format);
};


/* Local storage utilities */
Sports247.Utilities.LocalStorage = {};
Sports247.Utilities.LocalStorage.getInt = function(key) {
	return (localStorage[key] === undefined) ? undefined : parseInt(localStorage[key]);
};
Sports247.Utilities.LocalStorage.getBool = function(key) {
	if (localStorage[key] === undefined) {
		return undefined;
	} else if (localStorage[key]) {
		return true;
	}
	return false;
};
Sports247.Utilities.LocalStorage.getDate = function(key) {
	return (localStorage[key] === undefined) ? undefined : new Date(localStorage[key]);
};
